/*
 * cpg-layout.h
 * This file is part of cpg-network
 *
 * Copyright (C) 2011 - Jesse van den Kieboom
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifndef __CPG_LAYOUT_H__
#define __CPG_LAYOUT_H__

#include <cpg-network/cpg-network.h>
#include <cpg-network/cpg-layoutable.h>

G_BEGIN_DECLS

#define CPG_TYPE_LAYOUT			(cpg_layout_get_type ())
#define CPG_LAYOUT(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), CPG_TYPE_LAYOUT, CpgLayout))
#define CPG_LAYOUT_CONST(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), CPG_TYPE_LAYOUT, CpgLayout const))
#define CPG_LAYOUT_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), CPG_TYPE_LAYOUT, CpgLayoutClass))
#define CPG_IS_LAYOUT(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), CPG_TYPE_LAYOUT))
#define CPG_IS_LAYOUT_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), CPG_TYPE_LAYOUT))
#define CPG_LAYOUT_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), CPG_TYPE_LAYOUT, CpgLayoutClass))

typedef struct _CpgLayout		CpgLayout;
typedef struct _CpgLayoutClass		CpgLayoutClass;
typedef struct _CpgLayoutPrivate	CpgLayoutPrivate;

typedef enum
{
	CPG_LAYOUT_RELATION_ABOVE = 1 << 0,
	CPG_LAYOUT_RELATION_BELOW = 1 << 1,
	CPG_LAYOUT_RELATION_LEFT_OF = 1 << 2,
	CPG_LAYOUT_RELATION_RIGHT_OF = 1 << 3
} CpgLayoutRelation;

struct _CpgLayout
{
	/*< private >*/
	GObject parent;

	CpgLayoutPrivate *priv;
};

struct _CpgLayoutClass
{
	/*< private >*/
	GObjectClass parent_class;
};

GType      cpg_layout_get_type (void) G_GNUC_CONST;

CpgLayout *cpg_layout_new      (CpgNetwork        *network);

void       cpg_layout_add      (CpgLayout         *layout,
                                CpgLayoutable     *left,
                                CpgLayoutable     *right,
                                CpgLayoutRelation  relation);

void       cpg_layout_set      (CpgLayout         *layout,
                                CpgLayoutable     *layoutable,
                                gint               x,
                                gint               y);

G_END_DECLS

#endif /* __CPG_LAYOUT_H__ */
